@@ -8,7 +8,7 @@
  */
 
 
-#include <config.h>
+#include "config.h"
 
 #include <sys/types.h>
 
@@ -408,8 +408,10 @@ dname_to_string_r(const dname_type *dname, const dname_type *origin,
 		*buf = '\0';
 		return buf;
 	}
+	dst = buf;
 	if (dname->label_count == 1) {
-		strlcpy(buf, ".", sizeof(buf));
+		*dst++ = '.';
+		*dst = '\0';
 		return buf;
 	}
 	labels_to_convert = dname->label_count - 1;
@@ -420,7 +422,6 @@ dname_to_string_r(const dname_type *dname, const dname_type *origin,
 		absolute = 0;
 	}
 
-	dst = buf;
 	src = dname_name(dname);
 	for (i = 0; i < labels_to_convert; ++i) {
 		size_t len = label_length(src);
@@ -571,4 +572,5 @@ dname_make_wildcard(struct region *region,
 	*wildcard = new_dname;
 	return 0;
 }
-#endif
\ No newline at end of file
+#endif
+
