@@ -93,11 +93,11 @@ static rrtype_descriptor_type rrtype_descriptors[(RRTYPE_DESCRIPTORS_LENGTH+1)]
 	  { RDATA_ZF_TEXTS } },
 	/* 17 */
 	{ TYPE_RP, "RP", T_RP, 2, 2,
-	  { RDATA_WF_COMPRESSED_DNAME, RDATA_WF_COMPRESSED_DNAME },
+	  { RDATA_WF_UNCOMPRESSED_DNAME, RDATA_WF_UNCOMPRESSED_DNAME },
 	  { RDATA_ZF_DNAME, RDATA_ZF_DNAME } },
 	/* 18 */
 	{ TYPE_AFSDB, "AFSDB", T_AFSDB, 2, 2,
-	  { RDATA_WF_SHORT, RDATA_WF_COMPRESSED_DNAME },
+	  { RDATA_WF_SHORT, RDATA_WF_UNCOMPRESSED_DNAME },
 	  { RDATA_ZF_SHORT, RDATA_ZF_DNAME } },
 	/* 19 */
 	{ TYPE_X25, "X25", T_X25, 1, 1,
@@ -109,7 +109,7 @@ static rrtype_descriptor_type rrtype_descriptors[(RRTYPE_DESCRIPTORS_LENGTH+1)]
 	  { RDATA_ZF_TEXT, RDATA_ZF_TEXT } },
 	/* 21 */
 	{ TYPE_RT, "RT", T_RT, 2, 2,
-	  { RDATA_WF_SHORT, RDATA_WF_COMPRESSED_DNAME },
+	  { RDATA_WF_SHORT, RDATA_WF_UNCOMPRESSED_DNAME },
 	  { RDATA_ZF_SHORT, RDATA_ZF_DNAME } },
 	/* 22 */
 	{ TYPE_NSAP, "NSAP", T_NSAP, 1, 1,
@@ -402,6 +402,39 @@ static rrtype_descriptor_type rrtype_descriptors[(RRTYPE_DESCRIPTORS_LENGTH+1)]
 	    RDATA_ZF_TEXT, RDATA_ZF_TEXT, RDATA_ZF_TEXT, RDATA_ZF_TEXT,
 	    RDATA_ZF_TEXT, RDATA_ZF_TEXT, RDATA_ZF_TEXT, RDATA_ZF_TEXT,
 	    RDATA_ZF_TEXT, RDATA_ZF_TEXT, RDATA_ZF_TEXT, RDATA_ZF_TEXT } },
+	/* 100 */
+	{ 100, NULL, T_UTYPE, 1, 1, { RDATA_WF_BINARY }, { RDATA_ZF_UNKNOWN } },
+	/* 101 */
+	{ 101, NULL, T_UTYPE, 1, 1, { RDATA_WF_BINARY }, { RDATA_ZF_UNKNOWN } },
+	/* 102 */
+	{ 102, NULL, T_UTYPE, 1, 1, { RDATA_WF_BINARY }, { RDATA_ZF_UNKNOWN } },
+	/* 103 */
+	{ 103, NULL, T_UTYPE, 1, 1, { RDATA_WF_BINARY }, { RDATA_ZF_UNKNOWN } },
+	/* 104 */
+	{ TYPE_NID, "NID", T_NID, 2, 2,
+	  { RDATA_WF_SHORT, RDATA_WF_ILNP64 },
+	  { RDATA_ZF_SHORT, RDATA_ZF_ILNP64 } },
+	/* 105 */
+	{ TYPE_L32, "L32", T_L32, 2, 2,
+	  { RDATA_WF_SHORT, RDATA_WF_A },
+	  { RDATA_ZF_SHORT, RDATA_ZF_A } },
+	/* 106 */
+	{ TYPE_L64, "L64", T_L64, 2, 2,
+	  { RDATA_WF_SHORT, RDATA_WF_ILNP64 },
+	  { RDATA_ZF_SHORT, RDATA_ZF_ILNP64 } },
+	/* 107 */
+	{ TYPE_LP, "LP", T_LP, 2, 2,
+	  { RDATA_WF_SHORT, RDATA_WF_UNCOMPRESSED_DNAME },
+	  { RDATA_ZF_SHORT, RDATA_ZF_DNAME } },
+#ifdef DRAFT_RRTYPES
+	/* 108 */
+	{ TYPE_EUI48, "EUI48", T_EUI48, 1, 1,
+	  { RDATA_WF_EUI48 }, { RDATA_ZF_EUI48 } },
+	/* 109 */
+	{ TYPE_EUI64, "EUI64", T_EUI64, 1, 1,
+	  { RDATA_WF_EUI64 }, { RDATA_ZF_EUI64 } },
+#endif
+
 	/* 32769 */
 	{ TYPE_DLV, "DLV", T_DLV, 4, 4,
 	  { RDATA_WF_SHORT, RDATA_WF_BYTE, RDATA_WF_BYTE, RDATA_WF_BINARY },
