@@ -135,9 +135,20 @@ typedef enum nsd_rc nsd_rc_type;
 #define TYPE_DHCID	49	/* RFC4701 DHCP information */
 #define TYPE_NSEC3	50	/* NSEC3, secure denial, prevents zonewalking */
 #define TYPE_NSEC3PARAM 51	/* NSEC3PARAM at zone apex nsec3 parameters */
+#define TYPE_TLSA	52	/* RFC 6698 */
 
 #define TYPE_SPF        99      /* RFC 4408 */
 
+#define TYPE_NID        104     /* RFC 6742 */
+#define TYPE_L32        105     /* RFC 6742 */
+#define TYPE_L64        106     /* RFC 6742 */
+#define TYPE_LP         107     /* RFC 6742 */
+
+#ifdef DRAFT_RRTYPES
+#define TYPE_EUI48      108     /* draft-jabley-dnsext-eui48-eui64-rrtypes */
+#define TYPE_EUI64      109     /* draft-jabley-dnsext-eui48-eui64-rrtypes */
+#endif
+
 #define TYPE_TSIG	250
 #define TYPE_IXFR	251
 #define TYPE_AXFR	252
@@ -151,7 +162,7 @@ typedef enum nsd_rc nsd_rc_type;
 #define MAXLABELLEN	63
 #define MAXDOMAINLEN	255
 
-#define MAXRDATALEN	64      /* This is more than enough, think multiple TXT.  */
+#define MAXRDATALEN	64      /* This is more than enough, think multiple TXT. */
 #define MAX_RDLENGTH	65535
 
 /* Maximum size of a single RR.  */
@@ -160,6 +171,10 @@ typedef enum nsd_rc nsd_rc_type;
 
 #define IP4ADDRLEN	(32/8)
 #define IP6ADDRLEN	(128/8)
+#ifdef DRAFT_RRTYPES
+#define EUI48ADDRLEN	(48/8)
+#define EUI64ADDRLEN	(64/8)
+#endif
 
 /*
  * The different types of RDATA wireformat data.
@@ -169,17 +184,24 @@ enum rdata_wireformat
 	RDATA_WF_COMPRESSED_DNAME,   /* Possibly compressed domain name.  */
 	RDATA_WF_UNCOMPRESSED_DNAME, /* Uncompressed domain name.  */
 	RDATA_WF_LITERAL_DNAME,      /* Literal (not downcased) dname.  */
-	RDATA_WF_BYTE,		     /* 8-bit integer.  */
-	RDATA_WF_SHORT,		     /* 16-bit integer.  */
-	RDATA_WF_LONG,		     /* 32-bit integer.  */
-	RDATA_WF_TEXT,		     /* Text string.  */
-	RDATA_WF_TEXTS,		     /* Text string sequence.  */
-	RDATA_WF_A,		     /* 32-bit IPv4 address.  */
-	RDATA_WF_AAAA,		     /* 128-bit IPv6 address.  */
-	RDATA_WF_BINARY, 	     /* Binary data (unknown length).  */
+	RDATA_WF_BYTE,               /* 8-bit integer.  */
+	RDATA_WF_SHORT,              /* 16-bit integer.  */
+	RDATA_WF_LONG,               /* 32-bit integer.  */
+	RDATA_WF_TEXT,               /* Text string.  */
+	RDATA_WF_TEXTS,              /* Text string sequence.  */
+	RDATA_WF_A,                  /* 32-bit IPv4 address.  */
+	RDATA_WF_AAAA,               /* 128-bit IPv6 address.  */
+	RDATA_WF_BINARY,             /* Binary data (unknown length).  */
 	RDATA_WF_BINARYWITHLENGTH,   /* Binary data preceded by 1 byte length */
-	RDATA_WF_APL,		     /* APL data.  */
-	RDATA_WF_IPSECGATEWAY	     /* IPSECKEY gateway ip4, ip6 or dname. */
+	RDATA_WF_APL,                /* APL data.  */
+	RDATA_WF_IPSECGATEWAY,       /* IPSECKEY gateway ip4, ip6 or dname. */
+#ifdef DRAFT_RRTYPES
+	RDATA_WF_ILNP64,             /* 64-bit uncompressed IPv6 address.  */
+	RDATA_WF_EUI48,	             /* 48-bit address.  */
+	RDATA_WF_EUI64               /* 64-bit address.  */
+#else
+	RDATA_WF_ILNP64              /* 64-bit uncompressed IPv6 address.  */
+#endif
 };
 typedef enum rdata_wireformat rdata_wireformat_type;
 
@@ -213,6 +235,11 @@ enum rdata_zoneformat
 	RDATA_ZF_NXT,		/* NXT type bitmap.  */
 	RDATA_ZF_NSEC,		/* NSEC type bitmap.  */
 	RDATA_ZF_LOC,		/* Location data.  */
+	RDATA_ZF_ILNP64,	/* 64-bit uncompressed IPv6 address.  */
+#ifdef DRAFT_RRTYPES
+	RDATA_ZF_EUI48,		/* EUI48 address.  */
+	RDATA_ZF_EUI64,		/* EUI64 address.  */
+#endif
 	RDATA_ZF_UNKNOWN	/* Unknown data.  */
 };
 typedef enum rdata_zoneformat rdata_zoneformat_type;
@@ -233,9 +260,13 @@ typedef struct rrtype_descriptor rrtype_descriptor_type;
  * Indexed by type.  The special type "0" can be used to get a
  * descriptor for unknown types (with one binary rdata).
  *
- * spf + 1
+ * EUI64 + 1
  */
-#define RRTYPE_DESCRIPTORS_LENGTH  (TYPE_SPF + 1)
+#ifdef DRAFT_RRTYPES
+#define RRTYPE_DESCRIPTORS_LENGTH  (TYPE_EUI64 + 1)
+#else
+#define RRTYPE_DESCRIPTORS_LENGTH  (TYPE_LP + 1)
+#endif
 rrtype_descriptor_type *rrtype_descriptor_by_name(const char *name);
 rrtype_descriptor_type *rrtype_descriptor_by_type(uint16_t type);
 
