@@ -7,7 +7,7 @@
  *
  */
 
-#include <config.h>
+#include "config.h"
 
 #include <sys/types.h>
 
@@ -20,7 +20,6 @@
 
 #include "namedb.h"
 
-
 static domain_type *
 allocate_domain_info(domain_table_type *table,
 		     const dname_type *dname,
