@@ -12,6 +12,7 @@
 #include <sys/types.h>
 #include <sys/param.h>
 #include <sys/socket.h>
+#include <sys/uio.h>
 #include <sys/wait.h>
 
 #include <netinet/in.h>
@@ -33,6 +34,8 @@
 #define SHUT_WR 1
 #endif
 
+#include <openssl/rand.h>
+
 #include "axfr.h"
 #include "namedb.h"
 #include "netio.h"
@@ -41,6 +44,8 @@
 #include "difffile.h"
 #include "nsec3.h"
 #include "ipc.h"
+#include "lookup3.h"
+#include "rrl.h"
 
 /*
  * Data for the UDP handlers.
@@ -86,17 +91,17 @@ struct tcp_handler_data
 	 * data, including this structure.  This region is destroyed
 	 * when the connection is closed.
 	 */
-	region_type     *region;
+	region_type*		region;
 
 	/*
 	 * The global nsd structure.
 	 */
-	struct nsd      *nsd;
+	struct nsd*			nsd;
 
 	/*
 	 * The current query data for this TCP connection.
 	 */
-	query_type      *query;
+	query_type*			query;
 
 	/*
 	 * These fields are used to enable the TCP accept handlers
@@ -111,7 +116,7 @@ struct tcp_handler_data
 	 * AXFR, if we're done processing, or if we should discard the
 	 * query and connection.
 	 */
-	query_state_type query_state;
+	query_state_type	query_state;
 
 	/*
 	 * The bytes_transmitted field is used to remember the number
@@ -119,7 +124,7 @@ struct tcp_handler_data
 	 * packet.  The count includes the two additional bytes used
 	 * to specify the packet length on a TCP connection.
 	 */
-	size_t           bytes_transmitted;
+	size_t				bytes_transmitted;
 
 	/*
 	 * The number of queries handled by this specific TCP connection.
@@ -169,6 +174,8 @@ static void handle_tcp_writing(netio_type *netio,
  * Send all children the quit nonblocking, then close pipe.
  */
 static void send_children_quit(struct nsd* nsd);
+/* same, for shutdown time, waits for child to exit to avoid restart issues */
+static void send_children_quit_and_wait(struct nsd* nsd);
 
 /* set childrens flags to send NSD_STATS to them */
 #ifdef BIND8_STATS
@@ -353,7 +360,7 @@ int
 server_init(struct nsd *nsd)
 {
 	size_t i;
-#if defined(SO_REUSEADDR) || (defined(INET6) && (defined(IPV6_V6ONLY) || defined(IPV6_USE_MIN_MTU) || defined(IPV6_MTU)))
+#if defined(SO_REUSEADDR) || (defined(INET6) && (defined(IPV6_V6ONLY) || defined(IPV6_USE_MIN_MTU) || defined(IPV6_MTU) || defined(IP_TRANSPARENT)))
 	int on = 1;
 #endif
 
@@ -454,6 +461,15 @@ server_init(struct nsd *nsd)
 		}
 
 		/* Bind it... */
+		if (nsd->options->ip_transparent) {
+#ifdef IP_TRANSPARENT
+			if (setsockopt(nsd->udp[i].s, IPPROTO_IP, IP_TRANSPARENT, &on, sizeof(on)) < 0) {
+				log_msg(LOG_ERR, "setsockopt(...,IP_TRANSPARENT, ...) failed for udp: %s",
+					strerror(errno));
+			}
+#endif /* IP_TRANSPARENT */
+		}
+
 		if (bind(nsd->udp[i].s, (struct sockaddr *) nsd->udp[i].addr->ai_addr, nsd->udp[i].addr->ai_addrlen) != 0) {
 			log_msg(LOG_ERR, "can't bind udp socket: %s", strerror(errno));
 			return -1;
@@ -486,12 +502,39 @@ server_init(struct nsd *nsd)
 		}
 #endif /* SO_REUSEADDR */
 
-#if defined(INET6) && defined(IPV6_V6ONLY)
-		if (nsd->tcp[i].addr->ai_family == AF_INET6 &&
-		    setsockopt(nsd->tcp[i].s, IPPROTO_IPV6, IPV6_V6ONLY, &on, sizeof(on)) < 0)
-		{
-			log_msg(LOG_ERR, "setsockopt(..., IPV6_V6ONLY, ...) failed: %s", strerror(errno));
-			return -1;
+#if defined(INET6)
+		if (nsd->tcp[i].addr->ai_family == AF_INET6) {
+# if defined(IPV6_V6ONLY)
+			if (setsockopt(nsd->tcp[i].s, IPPROTO_IPV6, IPV6_V6ONLY,
+				&on, sizeof(on)) < 0) {
+				log_msg(LOG_ERR, "setsockopt(..., IPV6_V6ONLY, ...) failed: %s", strerror(errno));
+				return -1;
+			}
+# endif
+# if defined(IPV6_USE_MIN_MTU)
+			/*
+			 * Use minimum MTU to minimize delays learning working
+			 * PMTU when communicating through a tunnel.
+			 */
+			if (setsockopt(nsd->tcp[i].s,
+				       IPPROTO_IPV6, IPV6_USE_MIN_MTU,
+				       &on, sizeof(on)) < 0) {
+				log_msg(LOG_ERR, "setsockopt(..., IPV6_USE_MIN_MTU, ...) failed: %s", strerror(errno));
+				return -1;
+			}
+# elif defined(IPV6_MTU)
+			/*
+			 * On Linux, PMTUD is disabled by default for datagrams
+			 * so set the MTU equal to the MIN MTU to get the same.
+			 */
+			on = IPV6_MIN_MTU;
+			if (setsockopt(nsd->tcp[i].s, IPPROTO_IPV6, IPV6_MTU,
+				&on, sizeof(on)) < 0) {
+				log_msg(LOG_ERR, "setsockopt(..., IPV6_MTU, ...) failed: %s", strerror(errno));
+				return -1;
+			}
+			on = 1;
+# endif
 		}
 #endif
 		/* set it nonblocking */
@@ -502,6 +545,15 @@ server_init(struct nsd *nsd)
 		}
 
 		/* Bind it... */
+		if (nsd->options->ip_transparent) {
+#ifdef IP_TRANSPARENT
+			if (setsockopt(nsd->tcp[i].s, IPPROTO_IP, IP_TRANSPARENT, &on, sizeof(on)) < 0) {
+				log_msg(LOG_ERR, "setsockopt(...,IP_TRANSPARENT, ...) failed for tcp: %s",
+					strerror(errno));
+			}
+#endif /* IP_TRANSPARENT */
+		}
+
 		if (bind(nsd->tcp[i].s, (struct sockaddr *) nsd->tcp[i].addr->ai_addr, nsd->tcp[i].addr->ai_addrlen) != 0) {
 			log_msg(LOG_ERR, "can't bind tcp socket: %s", strerror(errno));
 			return -1;
@@ -524,6 +576,26 @@ server_init(struct nsd *nsd)
 int
 server_prepare(struct nsd *nsd)
 {
+#ifdef RATELIMIT
+	/* set secret modifier for hashing (udb ptr buckets and rate limits) */
+#ifdef HAVE_ARC4RANDOM
+	srandom(arc4random());
+	hash_set_raninit(arc4random());
+#else
+	uint32_t v = getpid() ^ time(NULL);
+	srandom((unsigned long)v);
+	if(RAND_status() && RAND_bytes((unsigned char*)&v, sizeof(v)) > 0)
+		hash_set_raninit(v);
+	else	hash_set_raninit(random());
+#endif
+	rrl_mmap_init(nsd->child_count, nsd->options->rrl_size,
+		nsd->options->rrl_ratelimit,
+		nsd->options->rrl_whitelist_ratelimit,
+		nsd->options->rrl_slip,
+		nsd->options->rrl_ipv4_prefix_length,
+		nsd->options->rrl_ipv6_prefix_length);
+#endif /* RATELIMIT */
+
 	/* Open the database... */
 	if ((nsd->db = namedb_open(nsd->dbfile, nsd->options, nsd->child_count)) == NULL) {
 		log_msg(LOG_ERR, "unable to open the database %s: %s",
@@ -570,8 +642,8 @@ server_start_children(struct nsd *nsd, region_type* region, netio_type* netio,
 	return restart_child_servers(nsd, region, netio, xfrd_sock_p);
 }
 
-static void
-close_all_sockets(struct nsd_socket sockets[], size_t n)
+void
+server_close_all_sockets(struct nsd_socket sockets[], size_t n)
 {
 	size_t i;
 
@@ -590,13 +662,13 @@ close_all_sockets(struct nsd_socket sockets[], size_t n)
  * Does not return.
  *
  */
-static void
+void
 server_shutdown(struct nsd *nsd)
 {
 	size_t i;
 
-	close_all_sockets(nsd->udp, nsd->ifs);
-	close_all_sockets(nsd->tcp, nsd->ifs);
+	server_close_all_sockets(nsd->udp, nsd->ifs);
+	server_close_all_sockets(nsd->tcp, nsd->ifs);
 	/* CHILD: close command channel to parent */
 	if(nsd->this_child && nsd->this_child->parent_fd != -1)
 	{
@@ -750,6 +822,9 @@ server_reload(struct nsd *nsd, region_type* server_region, netio_type* netio,
 			log_msg(LOG_ERR, "unable to reload the database: %s", strerror(errno));
 			exit(1);
 		}
+#if defined(NSEC3) && !defined(FULL_PREHASH)
+		prehash(nsd->db, 0);
+#endif
 	}
 	if(!diff_read_file(nsd->db, nsd->options, NULL, nsd->child_count)) {
 		log_msg(LOG_ERR, "unable to load the diff file: %s", nsd->options->difffile);
@@ -781,7 +856,7 @@ server_reload(struct nsd *nsd, region_type* server_region, netio_type* netio,
 
 	/* Start new child processes */
 	if (server_start_children(nsd, server_region, netio, xfrd_sock_p) != 0) {
-		send_children_quit(nsd);
+		send_children_quit(nsd); /* no wait required */
 		exit(1);
 	}
 
@@ -790,7 +865,7 @@ server_reload(struct nsd *nsd, region_type* server_region, netio_type* netio,
 		DEBUG(DEBUG_IPC,1, (LOG_INFO, "reload: ipc command from main %d", cmd));
 		if(cmd == NSD_QUIT) {
 			DEBUG(DEBUG_IPC,1, (LOG_INFO, "reload: quit to follow nsd"));
-			send_children_quit(nsd);
+			send_children_quit(nsd); /* no wait required */
 			exit(0);
 		}
 	}
@@ -808,7 +883,7 @@ server_reload(struct nsd *nsd, region_type* server_region, netio_type* netio,
 	/* Send quit command to parent: blocking, wait for receipt. */
 	do {
 		DEBUG(DEBUG_IPC,1, (LOG_INFO, "reload: ipc send quit to main"));
-		if (write_socket(cmdsocket, &cmd, sizeof(cmd)) == -1)
+		if (!write_socket(cmdsocket, &cmd, sizeof(cmd)))
 		{
 			log_msg(LOG_ERR, "problems sending command from reload %d to oldnsd %d: %s",
 				(int)nsd->pid, (int)old_pid, strerror(errno));
@@ -825,7 +900,7 @@ server_reload(struct nsd *nsd, region_type* server_region, netio_type* netio,
 		log_msg(LOG_ERR, "reload: could not wait for parent to quit: %s",
 			strerror(errno));
 	}
-	DEBUG(DEBUG_IPC,1, (LOG_INFO, "reload: ipc reply main %d %d", ret, cmd));
+	DEBUG(DEBUG_IPC,1, (LOG_INFO, "reload: ipc reply main %d %d", ret, (int)cmd));
 	if(cmd == NSD_QUIT) {
 		/* small race condition possible here, parent got quit cmd. */
 		send_children_quit(nsd);
@@ -964,6 +1039,10 @@ server_main(struct nsd *nsd)
 	pid_t xfrd_pid = -1;
 	sig_atomic_t mode;
 
+#ifdef RATELIMIT
+	rrl_init((nsd->this_child - nsd->children)/sizeof(nsd->children[0]));
+#endif
+
 	/* Ensure we are the main process */
 	assert(nsd->server_kind == NSD_SERVER_MAIN);
 
@@ -980,7 +1059,7 @@ server_main(struct nsd *nsd)
 
 	/* Start the child processes that handle incoming queries */
 	if (server_start_children(nsd, server_region, netio, &xfrd_listener.fd) != 0) {
-		send_children_quit(nsd);
+		send_children_quit(nsd); /* no wait required */
 		exit(1);
 	}
 	reload_listener.fd = -1;
@@ -1020,8 +1099,7 @@ server_main(struct nsd *nsd)
 					reload_listener.fd = -1;
 					reload_listener.event_types = NETIO_EVENT_NONE;
 					/* inform xfrd reload attempt ended */
-					if(!write_socket(xfrd_listener.fd,
-						&cmd, sizeof(cmd)) == -1) {
+					if(!write_socket(xfrd_listener.fd, &cmd, sizeof(cmd))) {
 						log_msg(LOG_ERR, "problems "
 						  "sending SOAEND to xfrd: %s",
 						  strerror(errno));
@@ -1142,7 +1220,7 @@ server_main(struct nsd *nsd)
 				close(reload_listener.fd);
 			}
 			/* only quit children after xfrd has acked */
-			send_children_quit(nsd);
+			send_children_quit(nsd); /* no wait required */
 
 			namedb_fd_close(nsd->db);
 			region_destroy(server_region);
@@ -1151,7 +1229,7 @@ server_main(struct nsd *nsd)
 			/* ENOTREACH */
 			break;
 		case NSD_SHUTDOWN:
-			send_children_quit(nsd);
+			send_children_quit_and_wait(nsd);
 			log_msg(LOG_WARNING, "signal received, shutting down...");
 			break;
 		case NSD_REAP_CHILDREN:
@@ -1174,9 +1252,9 @@ server_main(struct nsd *nsd)
 	/* Truncate the pid file.  */
 	if ((fd = open(nsd->pidfile, O_WRONLY | O_TRUNC, 0644)) == -1) {
 		log_msg(LOG_ERR, "can not truncate the pid file %s: %s", nsd->pidfile, strerror(errno));
+	} else {
+		close(fd);
 	}
-	close(fd);
-
 	/* Unlink it if possible... */
 	unlinkpid(nsd->pidfile);
 
@@ -1234,10 +1312,10 @@ server_child(struct nsd *nsd)
 	DEBUG(DEBUG_IPC, 2, (LOG_INFO, "child process started"));
 
 	if (!(nsd->server_kind & NSD_SERVER_TCP)) {
-		close_all_sockets(nsd->tcp, nsd->ifs);
+		server_close_all_sockets(nsd->tcp, nsd->ifs);
 	}
 	if (!(nsd->server_kind & NSD_SERVER_UDP)) {
-		close_all_sockets(nsd->udp, nsd->ifs);
+		server_close_all_sockets(nsd->udp, nsd->ifs);
 	}
 
 	if (nsd->this_child && nsd->this_child->parent_fd != -1) {
@@ -1320,8 +1398,11 @@ server_child(struct nsd *nsd)
 		/* Do we need to do the statistics... */
 		if (mode == NSD_STATS) {
 #ifdef BIND8_STATS
+			int p = nsd->st.period;
+			nsd->st.period = 1; /* force stats printout */
 			/* Dump the statistics */
 			bind8_stats(nsd);
+			nsd->st.period = p;
 #else /* !BIND8_STATS */
 			log_msg(LOG_NOTICE, "Statistics support not enabled at compile time.");
 #endif /* BIND8_STATS */
@@ -1369,6 +1450,20 @@ server_child(struct nsd *nsd)
 	server_shutdown(nsd);
 }
 
+static query_state_type
+server_process_query_udp(struct nsd *nsd, struct query *query)
+{
+#ifdef RATELIMIT
+	if(query_process(query, nsd) != QUERY_DISCARDED) {
+		if(rrl_process_query(query))
+			return rrl_slip(query);
+		else	return QUERY_PROCESSED;
+	}
+	return QUERY_DISCARDED;
+#else
+	return query_process(query, nsd);
+#endif
+}
 
 static void
 handle_udp(netio_type *ATTR_UNUSED(netio),
@@ -1413,18 +1508,23 @@ handle_udp(netio_type *ATTR_UNUSED(netio),
 		buffer_flip(q->packet);
 
 		/* Process and answer the query... */
-		if (server_process_query(data->nsd, q) != QUERY_DISCARDED) {
+		if (server_process_query_udp(data->nsd, q) != QUERY_DISCARDED) {
 #ifdef BIND8_STATS
 			if (RCODE(q->packet) == RCODE_OK && !AA(q->packet)) {
 				STATUP(data->nsd, nona);
-				ZTATUP(q->zone, nona);
+# ifdef USE_ZONE_STATS
+				if (q->zone)
+					ZTATUP(q->zone, nona);
+# endif
 			}
 
 # ifdef USE_ZONE_STATS
-			if (data->socket->addr->ai_family == AF_INET) {
+			if (q->zone) {
+			  if (data->socket->addr->ai_family == AF_INET) {
 				ZTATUP(q->zone, qudp);
-			} else if (data->socket->addr->ai_family == AF_INET6) {
+			  } else if (data->socket->addr->ai_family == AF_INET6) {
 				ZTATUP(q->zone, qudp6);
+			  }
 			}
 # endif
 #endif
@@ -1443,17 +1543,27 @@ handle_udp(netio_type *ATTR_UNUSED(netio),
 			if (sent == -1) {
 				log_msg(LOG_ERR, "sendto failed: %s", strerror(errno));
 				STATUP(data->nsd, txerr);
-				ZTATUP(q->zone, txerr);
+
+#ifdef USE_ZONE_STATS
+				if (q->zone)
+					ZTATUP(q->zone, txerr);
+#endif
 			} else if ((size_t) sent != buffer_remaining(q->packet)) {
 				log_msg(LOG_ERR, "sent %d in place of %d bytes", sent, (int) buffer_remaining(q->packet));
 #ifdef BIND8_STATS
 			} else {
 				/* Account the rcode & TC... */
 				STATUP2(data->nsd, rcode, RCODE(q->packet));
-				ZTATUP2(q->zone, rcode, RCODE(q->packet));
+# ifdef USE_ZONE_STATS
+				if (q->zone)
+					ZTATUP2(q->zone, rcode, RCODE(q->packet));
+# endif
 				if (TC(q->packet)) {
 					STATUP(data->nsd, truncated);
-					ZTATUP(q->zone, truncated);
+# ifdef USE_ZONE_STATS
+					if (q->zone)
+						ZTATUP(q->zone, truncated);
+# endif
 				}
 #endif /* BIND8_STATS */
 			}
@@ -1665,12 +1775,16 @@ handle_tcp_reading(netio_type *netio,
 	    && !AA(data->query->packet))
 	{
 		STATUP(data->nsd, nona);
-		ZTATUP(data->query->zone, nona);
+# ifdef USE_ZONE_STATS
+		if (data->query->zone)
+			ZTATUP(data->query->zone, nona);
+# endif
 	}
 
 # ifdef USE_ZONE_STATS
+	if (data->query->zone) {
 #  ifndef INET6
-	ZTATUP(data->query->zone, ctcp);
+		ZTATUP(data->query->zone, ctcp);
 #  else
 	if (data->query->addr.ss_family == AF_INET) {
 		ZTATUP(data->query->zone, ctcp);
@@ -1678,6 +1792,7 @@ handle_tcp_reading(netio_type *netio,
 		ZTATUP(data->query->zone, ctcp6);
 	}
 #  endif
+	}
 # endif /* USE_ZONE_STATS */
 
 #endif /* BIND8_STATS */
@@ -1718,9 +1833,18 @@ handle_tcp_writing(netio_type *netio,
 	if (data->bytes_transmitted < sizeof(q->tcplen)) {
 		/* Writing the response packet length.  */
 		uint16_t n_tcplen = htons(q->tcplen);
+#ifdef HAVE_WRITEV
+		struct iovec iov[2];
+		iov[0].iov_base = (uint8_t*)&n_tcplen + data->bytes_transmitted;
+		iov[0].iov_len = sizeof(n_tcplen) - data->bytes_transmitted; 
+		iov[1].iov_base = buffer_begin(q->packet);
+		iov[1].iov_len = buffer_limit(q->packet);
+		sent = writev(handler->fd, iov, 2);
+#else /* HAVE_WRITEV */
 		sent = write(handler->fd,
 			     (const char *) &n_tcplen + data->bytes_transmitted,
 			     sizeof(n_tcplen) - data->bytes_transmitted);
+#endif /* HAVE_WRITEV */
 		if (sent == -1) {
 			if (errno == EAGAIN || errno == EINTR) {
 				/*
@@ -1750,11 +1874,13 @@ handle_tcp_writing(netio_type *netio,
 			return;
 		}
 
-		assert(data->bytes_transmitted == sizeof(q->tcplen));
+#ifdef HAVE_WRITEV
+		sent -= sizeof(n_tcplen);
+		/* handle potential 'packet done' code */
+		goto packet_could_be_done;
+#endif
 	}
 
-	assert(data->bytes_transmitted < q->tcplen + sizeof(q->tcplen));
-
 	sent = write(handler->fd,
 		     buffer_current(q->packet),
 		     buffer_remaining(q->packet));
@@ -1778,8 +1904,11 @@ handle_tcp_writing(netio_type *netio,
 		}
 	}
 
-	buffer_skip(q->packet, sent);
 	data->bytes_transmitted += sent;
+#ifdef HAVE_WRITEV
+  packet_could_be_done:
+#endif
+	buffer_skip(q->packet, sent);
 	if (data->bytes_transmitted < q->tcplen + sizeof(q->tcplen)) {
 		/*
 		 * Still more data to write when socket becomes
@@ -1954,9 +2083,8 @@ handle_tcp_accept(netio_type *netio,
 }
 
 static void
-send_children_quit(struct nsd* nsd)
+send_children_command(struct nsd* nsd, sig_atomic_t command, int timeout)
 {
-	sig_atomic_t command = NSD_QUIT;
 	size_t i;
 	assert(nsd->server_kind == NSD_SERVER_MAIN && nsd->this_child == 0);
 	for (i = 0; i < nsd->child_count; ++i) {
@@ -1970,6 +2098,11 @@ send_children_quit(struct nsd* nsd)
 					(int) command,
 					(int) nsd->children[i].pid,
 					strerror(errno));
+			} else if (timeout > 0) {
+				/* wait for reply */
+				(void)block_read(NULL,
+					nsd->children[i].child_fd,
+					&command, sizeof(command), timeout);
 			}
 			fsync(nsd->children[i].child_fd);
 			close(nsd->children[i].child_fd);
@@ -1978,6 +2111,19 @@ send_children_quit(struct nsd* nsd)
 	}
 }
 
+static void
+send_children_quit(struct nsd* nsd)
+{
+	send_children_command(nsd, NSD_QUIT, 0);
+}
+
+static void
+send_children_quit_and_wait(struct nsd* nsd)
+{
+	DEBUG(DEBUG_IPC, 1, (LOG_INFO, "send children quit and wait"));
+	send_children_command(nsd, NSD_QUIT_CHILD, 3);
+}
+
 #ifdef BIND8_STATS
 static void
 set_children_stats(struct nsd* nsd)
