@@ -1,13 +1,13 @@
 /*
  * zonec.c -- zone compiler.
  *
- * Copyright (c) 2001-2006, NLnet Labs. All rights reserved.
+ * Copyright (c) 2001-2011, NLnet Labs. All rights reserved.
  *
  * See LICENSE for the license.
  *
  */
 
-#include <config.h>
+#include "config.h"
 
 #include <assert.h>
 #include <fcntl.h>
@@ -37,10 +37,12 @@
 #include "rdata.h"
 #include "region-allocator.h"
 #include "util.h"
-#include "zparser.h"
 #include "options.h"
 #include "nsec3.h"
 
+#define ILNP_MAXDIGITS 4
+#define ILNP_NUMGROUPS 4
+
 const dname_type *error_dname;
 domain_type *error_domain;
 
@@ -383,21 +385,184 @@ zparser_conv_aaaa(region_type *region, const char *text)
 	return r;
 }
 
+
+uint16_t *
+zparser_conv_ilnp64(region_type *region, const char *text)
+{
+	uint16_t *r = NULL;
+	int ngroups, num;
+	unsigned long hex;
+	const char *ch;
+	int c;
+	char digits[ILNP_MAXDIGITS+1];
+	unsigned int ui[ILNP_NUMGROUPS];
+	uint16_t a[ILNP_NUMGROUPS];
+
+	ngroups = 1; /* Always at least one group */
+	num = 0;
+	for (ch = text; *ch != '\0'; ch++) {
+		if (*ch == ':') {
+			if (num <= 0) {
+				zc_error_prev_line("ilnp64: empty group of "
+					"digits is not allowed");
+				return NULL;
+			}
+			digits[num] = '\0';
+			hex = (unsigned long) strtol(digits, NULL, 16);
+			num = 0;
+			ui[ngroups - 1] = hex;
+			if (ngroups >= ILNP_NUMGROUPS) {
+				zc_error_prev_line("ilnp64: more than %d groups "
+					"of digits", ILNP_NUMGROUPS);
+				return NULL;
+			}
+			ngroups++;
+		} else {
+			/* Our grammar is stricter than the one accepted by
+			 * strtol. */
+			c = (int) *ch;
+			if (!isxdigit(c)) {
+				zc_error_prev_line("ilnp64: invalid "
+					"(non-hexadecimal) character %c", c);
+				return NULL;
+			}
+			if (num >= ILNP_MAXDIGITS) {
+				zc_error_prev_line("ilnp64: more than %d digits "
+					"in a group", ILNP_MAXDIGITS);
+				return NULL;
+			}
+			digits[num++] = *ch;
+		}
+	}
+	if (num <= 0) {
+		zc_error_prev_line("ilnp64: empty group of digits is not "
+			"allowed");
+		return NULL;
+	}
+	digits[num] = '\0';
+	hex = (unsigned long) strtol(digits, NULL, 16);
+	ui[ngroups - 1] = hex;
+	if (ngroups < 4) {
+		zc_error_prev_line("ilnp64: less than %d groups of digits",
+			ILNP_NUMGROUPS);
+		return NULL;
+	}
+
+	a[0] = htons(ui[0]);
+	a[1] = htons(ui[1]);
+	a[2] = htons(ui[2]);
+	a[3] = htons(ui[3]);
+	r = alloc_rdata_init(region, a, sizeof(a));
+	return r;
+}
+
+#ifdef DRAFT_RRTYPES
+static uint16_t *
+zparser_conv_eui48(region_type *region, const char *text)
+{
+	uint8_t nums[6];
+	uint16_t *r = NULL;
+	unsigned int a, b, c, d, e, f;
+	int l;
+
+	if (sscanf(text, "%2x-%2x-%2x-%2x-%2x-%2x%n",
+		&a, &b, &c, &d, &e, &f, &l) != 6 ||
+		l != (int)strlen(text)){
+		zc_error_prev_line("eui48: invalid rr");
+		return NULL;
+	}
+	nums[0] = (uint8_t)a;
+	nums[1] = (uint8_t)b;
+	nums[2] = (uint8_t)c;
+	nums[3] = (uint8_t)d;
+	nums[4] = (uint8_t)e;
+	nums[5] = (uint8_t)f;
+	r = alloc_rdata_init(region, nums, sizeof(nums));
+	return r;
+}
+
+static uint16_t *
+zparser_conv_eui64(region_type *region, const char *text)
+{
+	uint8_t nums[8];
+	uint16_t *r = NULL;
+	unsigned int a, b, c, d, e, f, g, h;
+	int l;
+	if (sscanf(text, "%2x-%2x-%2x-%2x-%2x-%2x-%2x-%2x%n",
+		&a, &b, &c, &d, &e, &f, &g, &h, &l) != 8 ||
+		l != (int)strlen(text)) {
+		zc_error_prev_line("eui64: invalid rr");
+		return NULL;
+	}
+	nums[0] = (uint8_t)a;
+	nums[1] = (uint8_t)b;
+	nums[2] = (uint8_t)c;
+	nums[3] = (uint8_t)d;
+	nums[4] = (uint8_t)e;
+	nums[5] = (uint8_t)f;
+	nums[6] = (uint8_t)g;
+	nums[7] = (uint8_t)h;
+	r = alloc_rdata_init(region, nums, sizeof(nums));
+	return r;
+}
+
+uint16_t *
+zparser_conv_eui(region_type *region, const char *text, size_t len)
+{
+	uint16_t *r = NULL;
+	int nnum, num;
+	const char* ch;
+
+	nnum = len/8;
+	num = 1;
+	for (ch = text; *ch != '\0'; ch++) {
+		int c = (int) *ch;
+		if (*ch == '-') {
+			num++;
+		} else if (!isxdigit(c)) {
+			zc_error_prev_line("eui%u: invalid (non-hexadecimal) "
+				"character %c", (unsigned) len, c);
+			return NULL;
+		}
+	}
+	if (num != nnum) {
+		zc_error_prev_line("eui%u: wrong number of hex numbers",
+			(unsigned) len);
+		return NULL;
+	}
+
+	switch (len) {
+		case 48:
+			r = zparser_conv_eui48(region, text);
+			break;
+		case 64:
+			r = zparser_conv_eui64(region, text);
+		break;
+		default:
+			zc_error_prev_line("eui%u: invalid length",
+				(unsigned) len);
+			return NULL;
+			break;
+	}
+	return r;
+}
+#endif
+
 uint16_t *
 zparser_conv_text(region_type *region, const char *text, size_t len)
 {
 	uint16_t *r = NULL;
+	uint8_t *p;
 
 	if (len > 255) {
 		zc_error_prev_line("text string is longer than 255 characters,"
 				   " try splitting it into multiple parts");
-	} else {
-		uint8_t *p;
-		r = alloc_rdata(region, len + 1);
-		p = (uint8_t *) (r + 1);
-		*p = len;
-		memcpy(p + 1, text, len);
+		len = 255;
 	}
+	r = alloc_rdata(region, len + 1);
+	p = (uint8_t *) (r + 1);
+	*p = len;
+	memcpy(p + 1, text, len);
 	return r;
 }
 
@@ -688,7 +853,7 @@ zparser_conv_loc(region_type *region, char *str)
 				zc_error_prev_line("space expected after seconds");
 				return NULL;
 			}
-
+			/* No need for precision specifiers, it's a double */
 			if (sscanf(start, "%lf", &d) != 1) {
 				zc_error_prev_line("error parsing seconds");
 			}
@@ -1162,7 +1327,10 @@ process_rr(void)
 
 	/* We only support IN class */
 	if (rr->klass != CLASS_IN) {
-		zc_error_prev_line("only class IN is supported");
+		if(zone->opts && zone->opts->request_xfr)
+			zc_warning_prev_line("only class IN is supported");
+		else
+			zc_error_prev_line("only class IN is supported");
 		return 0;
 	}
 
@@ -1198,9 +1366,12 @@ process_rr(void)
 		 * This is a SOA record, start a new zone or continue
 		 * an existing one.
 		 */
-		if (rr->owner->is_apex)
-			zc_error_prev_line("this SOA record was already encountered");
-		else if (rr->owner == parser->default_apex) {
+		if (rr->owner->is_apex) {
+			if(zone->opts && zone->opts->request_xfr)
+				zc_warning_prev_line("this SOA record was already encountered");
+			else
+				zc_error_prev_line("this SOA record was already encountered");
+		} else if (rr->owner == parser->default_apex) {
 			zone->apex = rr->owner;
 			rr->owner->is_apex = 1;
 		}
@@ -1212,7 +1383,10 @@ process_rr(void)
 	if (!dname_is_subdomain(domain_dname(rr->owner),
 				domain_dname(zone->apex)))
 	{
-		zc_error_prev_line("out of zone data");
+		if(zone->opts && zone->opts->request_xfr)
+			zc_warning_prev_line("out of zone data");
+		else
+			zc_error_prev_line("out of zone data");
 		return 0;
 	}
 
@@ -1257,21 +1431,33 @@ process_rr(void)
 	}
 
 	if(rr->type == TYPE_DNAME && rrset->rr_count > 1) {
-		zc_error_prev_line("multiple DNAMEs at the same name");
+		if(zone->opts && zone->opts->request_xfr)
+			zc_warning_prev_line("multiple DNAMEs at the same name");
+		else
+			zc_error_prev_line("multiple DNAMEs at the same name");
 	}
 	if(rr->type == TYPE_CNAME && rrset->rr_count > 1) {
-		zc_error_prev_line("multiple CNAMEs at the same name");
+		if(zone->opts && zone->opts->request_xfr)
+			zc_warning_prev_line("multiple CNAMEs at the same name");
+		else
+			zc_error_prev_line("multiple CNAMEs at the same name");
 	}
 	if((rr->type == TYPE_DNAME && domain_find_rrset(rr->owner, zone, TYPE_CNAME))
 	 ||(rr->type == TYPE_CNAME && domain_find_rrset(rr->owner, zone, TYPE_DNAME))) {
-		zc_error_prev_line("DNAME and CNAME at the same name");
+		if(zone->opts && zone->opts->request_xfr)
+			zc_warning_prev_line("DNAME and CNAME at the same name");
+		else
+			zc_error_prev_line("DNAME and CNAME at the same name");
 	}
 	if(domain_find_rrset(rr->owner, zone, TYPE_CNAME) &&
 		domain_find_non_cname_rrset(rr->owner, zone)) {
-		zc_error_prev_line("CNAME and other data at the same name");
+		if(zone->opts && zone->opts->request_xfr)
+			zc_warning_prev_line("CNAME and other data at the same name");
+		else
+			zc_error_prev_line("CNAME and other data at the same name");
 	}
 
-	if (rr->type == TYPE_RRSIG && rr_rrsig_type_covered(rr) == TYPE_SOA) {
+	if (rr->type == TYPE_RRSIG && rr_rrsig_type_covered(rr) == TYPE_DNSKEY) {
 		rrset->zone->is_secure = 1;
 	}
 
@@ -1287,7 +1473,10 @@ process_rr(void)
 		}
 	}
 	else if (rr->type == TYPE_SOA) {
-		zc_error_prev_line("duplicate SOA record discarded");
+		if(zone->opts && zone->opts->request_xfr)
+			zc_warning_prev_line("duplicate SOA record discarded");
+		else
+			zc_error_prev_line("duplicate SOA record discarded");
 		--rrset->rr_count;
 	}
 
